$NOLIST
$NOSB
$NOMOD51
$INCLUDE (REG152.ASM)
$LIST

;*********************************************************************
;THIS PROGRAM IS WRITTEN TO START A TRANSMISSION IN SDLC MODE UNDER DMA 
;CONTROL AND ASSUMES SOME TRANSMISSIONS WILL BE IN ERROR, THUS THE NAME 
;Transmit Error Under Dma Control (TVUDC1).
;
;REQUIREMENTS:
;           NONE
;
;THE SOURCE OF DATA TO BE TRANSMITTED IS IN EXTERNAL RAM 
;THE STARTING ADDRESS OF THE DATA IS DEFINED BY:
;           MSG_SOURCE (CAN BE UPTO 16 BITS)
;
;THE LENGTH OF OF THE PACKET TO BE TRANSMITTED IS DEFINED BY:
;           MSG_LENGTH (CAN BE UPTO 16 BITS)
;
;INTERNAL RAM LOCATIONS USED AND MODIFIED:
;           0FF TO 0FA  UR_COUNTER
;
;SFRS USED AND MODIFIED:
;           ACC
;           BAUD
;           BCRH0
;           BCRL0
;           DARL0
;           DCON0
;           GMOD
;           IE
;           IEN1
;           IFS
;           P1
;           SARH0
;           SARL0
;           TFIFO
;           TSTAT
;
;REGISTERS USED AND MODIFIED:
;           R0
;           R1
;           R7
;
;*********************************************************************

$EJ

;*********************************************************************
;THIS PROGRAM CAN BE BROKEN INTO 4 SEPARATE SECTIONS.  THESE ARE: THE
;INTERRUPT VECTORS (LOCATIONS 0 TO 0FFH), GSC INTERRUPT SERVICE
;SOFTWARE (LOCATIONS 100H TO 1FFH), GSC AND DMA INITIALIZATION SOFTWARE 
;(LOCATIONS 200H TO 2FFH), AND THOSE SECTIONS OF CODE NEEDED TO MAKE 
;THE PROGRAM FUNCTION (LOCATIONS 300H TO 3FFH).  THIS SHOULD ALLOW A 
;USER TO PULL OUT ONLY THE SECTIONS THAT ARE NEEDED TO COMPLETE A 
;SPECIFIC APPLICATION.
;*********************************************************************

;*********************************************************************
;SINCE SDLC WAS CHOSEN AND THE GSC IS UNDER DMA CONTROL AN UNDERRUN ERROR 
;(UR) (AND ONLY AN UR ERROR) CAN OCCUR.  BECAUSE AN ERROR CAN OCCUR, THE 
;PROGRAM, TVUDC1.SRC HAS BEEN MODIFIED TO HANDLE THE UR.
;*********************************************************************
;ALL OF THE CODE REQUIRED JUST TO MAKE THE PROGRAM FLOW PROPERLY IS 
;LOCATED ABOVE LOCATION 300H.  ALL OF THE CODE NEEDED TO SERVICE THE GSC 
;IS LOCATED BETWEEN LOCATIONS 100H TO 200H (EXCEPT FOR THE INTERRUPT 
;VECTORS THEMSELVES).  THE CODE USED TO BEGIN A TRANSMISSION, TO 
;INITIALIZE THE GSC AND TO INITIALIZE THE DMA CHANNEL IS IN LOCATIONS 
;200H TO 300H.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN TO GO ALONG WITH THE INTEL FAE TRAINING
;MATERIAL FOR THE 8XC152 INTERMEDIATE TRAINING (JAN 1987) BUT HOPEFULLY,
;IS SIMPLE ENOUGH FOR USERS WHO HAVE NOT TAKEN THE TRAINING COURSE TO 
;UNDERSTAND. THE LETTERS IN PARENTHESES AND FOLLOWED BY A HYPHEN, (A) -, 
;REFER TO THE LETTERS IN FLOW CHARTS GIVEN DURING THE FAE TRAINING
;*********************************************************************

$EJ
UR_COUNTER          DATA    0FFH    ;RAM LOCATIONS 0F9H TO 0FFH ARE USED
                                    ;TO KEEP A LOG OF THE # OF UR ERRORS

MESSAGE_SOURCE      EQU     100H    ;STARTING LOCATION OF DATA
MESSAGE_LENGTH      EQU     10H     ;LENGTH OF PACKET TO TRANSMIT

ORG 0
    JMP INIT1

ORG 43H
XMIT_VALID:
    JMP VALID_XMIT

ORG 4BH
XMIT_ERROR:
    JMP ERROR_XMIT

$EJ
ORG 100H
END_OF_MESSAGE:
;*********************************************************************
; DISABLE TRANSMIT VALID INTERRUPT
;*********************************************************************
    CLR EGSTV                       ;CLEAR INTERRUPT

;*********************************************************************
; REINITIALIZE MESSAGE POINTER
;*********************************************************************

    MOV SARL0,#LOW (MESSAGE_SOURCE) 
    MOV SARH0,#HIGH (MESSAGE_SOURCE)    
                                    ;RE-INITIALIZE SOURCE POINTER

END_MESSAGE_SERVICE:
;*********************************************************************
; RETURN TO MAIN PROGRAM LOOP
;*********************************************************************
    RETI

ERROR_SERVICE:
;*********************************************************************
; SINCE UNDERRUN IS THE ONLY TYPE OF ERROR THAT CAN OCCUR, THE CHECK
; FOR UR (A) AND THE ASSOCIATED TASKS (B,C) SHOWN IN THE FAE FLOW 
; CHARTS ARE NOT NECESSARY.
;*********************************************************************
; (D,E) - FORCE TXD AND DEN LOW THEN HIGH
;*********************************************************************

    CLR P1.1
    CLR P1.2
    SETB P1.1
    SETB P1.2

;*********************************************************************
; STOP DMA CHANNEL (THIS TASK IS NOT IN THE FLOW CHARTS BUT SHOULD BE)
;*********************************************************************
    ANL DCON0,#0FEH                 ;CLEAR GO BIT

;*********************************************************************
; (F) - LOG FAILURE
;*********************************************************************

    MOV R1,#UR_COUNTER
    CALL INCREMENT_COUNTER

;*********************************************************************
; (G) - RE-INITIALIZE DMA
;*********************************************************************
    MOV SARL0,#LOW (MESSAGE_SOURCE) 
    MOV SARH0,#HIGH (MESSAGE_SOURCE)    
                                    ;RE-INITIALIZE SOURCE POINTER

;*********************************************************************
;IN THIS EXAMPLE AN IMMEDIATE (CONSTANT) VALUE IS LOADED INTO THE BYTE
;COUNT REGISTERS.  THIS IS POSSIBLE BECAUSE FIXED MESSAGE LENGTHS ARE
;USED.  IF VARIABLE SIZED PACKETS ARE TO BE USED, INSTEAD OF AN CONSTANT
;VALUE BEING LOADED INTO THE BYTE COUNT REGISTERS, THE CALCULATED VALUE
;OF THE MESSAGE LENGTH WOULD BE LOADED.
;*********************************************************************
    MOV BCRH0,#HIGH (MESSAGE_LENGTH)
    MOV BCRL0,#LOW (MESSAGE_LENGTH)
                                    ;RE-INITIALIZE BYTE COUNTER

;*********************************************************************
; (H) - INITIALIZE TRANSMITTER
;*********************************************************************
    SETB TEN

    JNB TEN,$                       ;WAIT UNTIL TEN IS SET (TEN WILL NOT 
                                    ;BE SET IF A TRANSMISSIONS CRC HAS
                                    ;NOT YET COMPLETED BUT TEN MIGHT BE
                                    ;CLEARED BEFORE CRC COMPLETES)

;*********************************************************************
; (I,J) - START TRANSMISSION AND RETURN
;*********************************************************************
    ORL DCON0,#01                   ;SET GO BIT
    RETI

$EJ
ORG 200H
;*********************************************************************
; USER GENERIC GSC INITIALIZATION
;*********************************************************************
INIT2:
    MOV BAUD,#60                    ;INIT BAUD RATE
    MOV GMOD,#01H                   ;INIT FOR SDLC, 8-BIT ADDR
    MOV IFS,#80H                    ;IFS IS USED IN SDLC
    SETB DMA                        ;INIT GSC INTERRUPTS FOR DMA

    MOV DARL0,#TFIFO                ;DESTINATION OF DATA IS TFIFO
    MOV DCON0,#10011000B            ;INIT DMA FOR DEST=SFR, SOURCE=EXT RAM
                                    ;SERIAL PORT DEMAND MODE
    RET

;*********************************************************************
;DECIDE IF A TRANSMISSION IS WANTED
;*********************************************************************
TRANSMISSION_DECISION:
    CJNE A,#01,DO_NOT_TRANSMIT      ;TRANSMIT IF ACC=1

;*********************************************************************
; DECIDE IF TRANSMISSION IN PROGRESS
;*********************************************************************
    JB TEN,DO_NOT_TRANSMIT

    JB TEN,DO_NOT_TRANSMIT          ;SECOND TEST IS NEEDED IF TRANSMIT
                                    ;INTERRUPT OCCURS ONE CYCLE PRIOR TO
                                    ;FIRST TEST.  THE INTERRUPT (ERROR)
                                    ;WILL CLEAR TEN FOR FIRST TEST BUT
                                    ;START TRANSMISSION AND WOULD RETURN
                                    ;TO THE NEXT INSTRUCTION WHICH COULD
                                    ;APPEND THE NEW TRANSMISSION. 

;*********************************************************************
; (C) - REINITIALIZE MESSAGE COUNTER 
;*********************************************************************
;THIS WAS MOVED FROM THE END-OF-MESSAGE ROUTINE (GSC TRANSMIT VALID 
;RESPONSE) BECAUSE THE LENGTH OF THE PACKET MAY NOT BE KNOWN UNTIL 
;TRANSMISSION IS ABOUT TO START.  INSTEAD, INITIALIZATION OF THE COUNTERS 
;WAS PLACED HERE, IN THE START-OF-MESSAGE ROUTINE.  THIS ALLOWS THE USER 
;TO CONFIGURE THE PACKET LENGTH WHEN TRANSMISSION STARTS, WHEN THE NUMBER 
;OF BYTES WILL BE KNOWN.
;*********************************************************************
;IN THIS EXAMPLE AN IMMEDIATE (CONSTANT) VALUE IS LOADED INTO THE BYTE
;COUNT REGISTERS.  THIS IS POSSIBLE BECAUSE FIXED MESSAGE LENGTHS ARE
;USED.  IF VARIABLE SIZED PACKETS ARE TO BE USED, INSTEAD OF AN CONSTANT
;VALUE BEING LOADED INTO THE BYTE COUNT REGISTERS, THE CALCULATED VALUE
;OF THE MESSAGE LENGTH WOULD BE LOADED.
;*********************************************************************
    MOV BCRH0,#HIGH (MESSAGE_LENGTH)
    MOV BCRL0,#LOW (MESSAGE_LENGTH)
                                    ;RE-INITIALIZE BYTE COUNTER

;*********************************************************************
; ENABLE TRANSMITTER AND DMA CHANNEL
;*********************************************************************
    SETB TEN

    JNB TEN,$                       ;WAIT UNTIL TEN IS SET (TEN WILL NOT 
                                    ;BE SET IF A TRANSMISSIONS CRC HAS
                                    ;NOT YET COMPLETED BUT TEN MIGHT BE
                                    ;CLEARED BEFORE CRC COMPLETES)

    ORL DCON0,#01                   ;SET GO BIT

;*********************************************************************
; ENABLE INTERRUPTS
;*********************************************************************
    SETB EA 
    SETB EGSTE                      ;ENABLE ERROR INTERRUPT TO PREVENT
                                    ;PROGRAM FROM HANGING UP
    SETB EGSTV                      ;ENABLE TRANSMIT VALID INTERRUPT
    JMP MAIN

DO_NOT_TRANSMIT:
    JMP MAIN

$EJ
ORG 300H
;*********************************************************************
;START OF EXTRA CODE TO MAKE PROGRAM WORK
;*********************************************************************

INCREMENT_COUNTER:
    MOV A,@R1
    ADD A,#1                        ;INCREMENT LOWEST BYTE OF COUNT
    MOV @R1,A                       ;AND STORE
    DEC R1
    MOV A,@R1                       ;GET SECOND BYTE OF COUNTER, 
    ADDC A,#0                       ;INCREMENT IF CARRY FROM PREVIOUS
    MOV @R1,A                       ;ADD AND STORE RESULT
    DEC R1
    MOV A,@R1                       ;GET THIRD BYTE OF COUNTER,
    ADDC A,#0                       ;INCREMENT IF CARRY FROM PREVIOUS
    MOV @R1,A                       ;ADD AND STORE RESULT
    DEC R1
    MOV A,@R1                       ;GET FOURTH BYTE OF COUNTER,
    ADDC A,#0                       ;INCREMENT IF CARRY FROM PREVIOUS
    MOV @R1,A                       ;ADD AND STORE RESULT
    DEC R1
    MOV A,@R1                       ;GET FIFTH BYTE OF COUNTER,
    ADDC A,#0                       ;INCREMENT IF CARRY FROM PREVIOUS
    MOV @R1,A                       ;ADD AND STORE RESULT
    DEC R1
    MOV A,@R1                       ;GET SIXTH (LAST) BYTE OF COUNTER,
    ADDC A,#0                       ;INCREMENT IF CARRY FROM PREVIOUS
    MOV @R1,A                       ;ADD AND STORE RESULT
    JC COUNTER_OVERFLOW             ;IF CARRY GENERATED FROM LAST ADD
                                    ;THEN COUNTER HAS OVERFLOWED
    RET

COUNTER_OVERFLOW:
    MOV @R1,#0FFH                   ;IF OVERFLOW FILL BYTE 6 WITH 0FFH
    INC R1
    MOV @R1,#0FFH                   ;FILL BYTE 5 WITH 0FFH
    INC R1
    MOV @R1,#0FFH                   ;FILL BYTE 4 WITH 0FFH
    INC R1
    MOV @R1,#0FFH                   ;FILL BYTE 3 WITH 0FFH
    INC R1
    MOV @R1,#0FFH                   ;FILL BYTE 2 WITH 0FFH
    INC R1
    MOV @R1,#0FFH                   ;FILL BYTE 1 WITH 0FFH

COUNTER_SERVICE_LOOP:
    MOV A,@R1                       ;GET BYTE 1 OF COUNTER
    JNZ COUNTER_SERVICE_LOOP        ;LOOP UNTIL COUNTER IS
                                    ;SERVICED (UP TO USER TO
                                    ;IMPLEMENT)
    RET


VALID_XMIT:
    CALL END_OF_MESSAGE             ;EGSTV SERVICE ROUTINE

VALID_END:
    RET                             ;RETURN TO MAIN PROGRAM LOOP 


ERROR_XMIT:
;*********************************************************************
;THE ONLY TYPE OF ERROR THAT CAN OCCUR IN SDLC MODE UNDER DMA CONTROL
;IS AN UNDERRUN.  CONSEQUENTLY, AN UR IS ASSUMED TO BE THE ERROR TO
;RESPOND TO.
;*********************************************************************

    CALL ERROR_SERVICE
    RET

INIT1:
;*********************************************************************
; USER GENERIC INITIALIZATION - USER WOULD PLACE HIS INITIALIZATION
; SOFTWARE HERE
;*********************************************************************

    MOV R7,#7
    MOV R0,#UR_COUNTER
INIT_LOOP:
    MOV @R0,#0
    DEC R0
    DJNZ R7,INIT_LOOP               ;CLEAR ERROR COUNTER RAM LOCATIONS

    CALL INIT2                      ;GSC INITIALIZATION

MAIN:
;*********************************************************************
; USER MAIN PROGRAM LOOP
;*********************************************************************

;*********************************************************************
; LOAD TRANSMISSION DECISION CRITERIA INTO ACCUMULATOR
;*********************************************************************
    MOV A,PRBS                      ;OBTAIN A RANDOM #
    ANL A,#0FH                      ;MASK OFF UPPER NIBBLE
    JMP TRANSMISSION_DECISION

END
